import createError from 'http-errors'
import logger from './logger.js'

export function createPath (path, config) {
  const concatenatedPath = (config.prependPath || '') + path
  try {
    if (!config.rewrite) return concatenatedPath
    return config.rewrite(concatenatedPath)
  } catch (err) {
    logger.warn(`Could not rewrite path ${concatenatedPath}. Falling back to original path`)
    return concatenatedPath
  }
}

export async function pipe (proxyRes, res) {
  return new Promise((resolve, reject) => {
    let bytesWritten = 0

    if (proxyRes.statusCode >= 400) {
      reject(createError(proxyRes.statusCode, proxyRes.message))
    }

    function setHeaders () {
      if (!bytesWritten) {
        res.statusCode = proxyRes.statusCode
        for (const [key, value] of Object.entries(proxyRes.headers)) {
          res.setHeader(key, value)
        }
      }
    }

    function onDrain () {
      proxyRes.resume()
    }

    // pipe the response data from proxyRes to res
    proxyRes
      .on('data', buf => {
        setHeaders()
        bytesWritten += buf.length
        if (!res.write(buf)) {
          proxyRes.pause()
        }
      })
      .on('end', () => {
        setHeaders()
        // trailers are only there, when version is compliant
        // proxyRes.trailers contains headers that are sent at the end of the request
        if (proxyRes.trailers) res.addTrailers(proxyRes.trailers)
        res.end()
        resolve()
      })
      .on('close', () => {
        res.off('drain', onDrain)
      })

    res.on('drain', onDrain)
  })
}
