import parseurl from 'parseurl'
import * as proxy from '../proxy.js'
import * as http from '../http.js'
import finalhandler from 'finalhandler'
import createError, { HttpError } from 'http-errors'
import logger from '../logger.js'

function getFSPath (path) {
  const index = path.indexOf('node_modules')
  if (index < 0) return path
  return path.substr(0, 32)
}

function proxyRequest (req, res, { hostname, protocol, port, secure }) {
  return proxy.web(req, res, {
    hostname,
    protocol,
    port,
    headers: { 'proxied-by-ox-proxy': true },
    secure,
    onRes (req, res, proxyRes) {
      if (proxyRes.statusCode >= 400) {
        throw createError(proxyRes.statusCode, proxyRes.message)
      }
      return http.pipe(proxyRes, res)
    }
  })
}

export default function ({ urls, secure }) {
  const cache = new Map()

  return async (req, res, next) => {
    if (req.headers['proxied-by-ox-proxy']) return next()

    const { path } = parseurl(req)

    // two stage system
    // 1. if we have a known path, we can directly request the file from that system
    // 2. otherwise, request them in order

    const isFsFile = path.startsWith('/@fs')
    const pathToHandler = isFsFile ? getFSPath(path) : path

    if (cache.has(pathToHandler)) {
      logger.debug(`Found a handler for ${path} in cache`)
      try {
        const { hostname, protocol, port } = cache.get(pathToHandler)
        await proxyRequest(req, res, { hostname, protocol, port, secure })
        logger.debug(`Delivered ${path} by a cached handler`)
        return
      } catch (err) {
        logger.warn(`The handler for ${path} could not be used anymore.`)
        cache.delete(pathToHandler)
      }
    }

    logger.debug(`Try to find ${path} in frontends`)
    const errors = []
    for (const url of urls) {
      const { hostname, protocol, port } = url
      try {
        await proxyRequest(req, res, { hostname, protocol, port, secure })
        cache.set(pathToHandler, url)
        logger.debug(`Found ${path} at ${protocol}//${hostname}:${port}`)
        return
      } catch (err) {
        errors.push(err)
      }
    }

    const err = errors.reduce((memo, err) => {
      // keep the first non-http error and forward it
      if (!(memo instanceof HttpError)) return memo
      // otherwise, use the last http error
      return err
    }, new createError.NotFound())

    const handle = finalhandler(req, res)
    if (err instanceof HttpError) {
      // in case of http-error the proxy explicitedly received a error code
      logger.warn(`Could not find ${path} in frontends`)
      return handle(err)
    }
    logger.error(err)
    return handle()
  }
}
