import { describe, it, expect, afterEach, jest } from '@jest/globals'
import path from 'path'
import vitePluginProxy from '../../src/index'
import vitePluginOxManifests from '@open-xchange/vite-plugin-ox-manifests'
import { closeAllServers, createExpressServer, createViteServer, getPort } from '../util'
import axios from 'axios'
import compression from 'compression'

const PORT = getPort()
const SERVER_PORT = getPort()
const SERVER_PORT2 = getPort()
const __dirname = path.dirname(new URL(import.meta.url).pathname)

describe('frontend uris', function () {
  afterEach(async function () {
    await closeAllServers()
  })

  it('works', async function () {
    await createViteServer({
      root: __dirname,
      logLevel: 'silent',
      plugins: [vitePluginProxy({
        frontendURIs: [`http://localhost:${SERVER_PORT}`]
      })]
    }, PORT)

    const app = await createExpressServer(SERVER_PORT)
    app.get('/other.js', (req, res) => {
      res.send('console.log("Hello world")')
    })

    const { data: index } = await axios({ baseURL: `http://localhost:${PORT}/index.js` })
    expect(index).toEqual('window.test = \'foo\'\n')

    const { data: other } = await axios({ baseURL: `http://localhost:${PORT}/other.js` })
    expect(other).toEqual('console.log("Hello world")')
  })

  it('serves merged manifest.json', async function () {
    await createViteServer({
      root: __dirname,
      logLevel: 'silent',
      plugins: [vitePluginProxy({
        frontendURIs: [`http://localhost:${SERVER_PORT}`]
      }), vitePluginOxManifests({
        // no config
      })]
    }, PORT)

    const app = await createExpressServer(SERVER_PORT)
    app.get('/manifests', (req, res) => {
      res.json([{ namespace: 'other' }])
    })

    const { data } = await axios({ baseURL: `http://localhost:${PORT}/manifests` })
    expect(data).toEqual([{
      namespace: 'other'
    }, {
      namespace: 'index',
      path: 'index'
    }])
  })

  it('overrides manifest.json with local manifests', async function () {
    await createViteServer({
      root: __dirname,
      logLevel: 'silent',
      plugins: [vitePluginProxy({
        // this has to be thought of as a remote UI with the same files
        frontendURIs: [`http://localhost:${SERVER_PORT}`]
      }), vitePluginOxManifests({
        // no config
      })]
    }, PORT)

    const app = await createExpressServer(SERVER_PORT)
    app.get('/manifests', (req, res) => {
      res.json([{ namespace: 'index', path: 'index', requires: 'chat' }])
    })

    const { data } = await axios({ baseURL: `http://localhost:${PORT}/manifests` })
    expect(data).toEqual([{
      namespace: 'index',
      path: 'index'
    }])
  })

  it('overrides remote files with local files', async function () {
    await createViteServer({
      root: __dirname,
      logLevel: 'silent',
      plugins: [vitePluginProxy({
        // this has to be thought of as a remote UI with the same files
        frontendURIs: [`http://localhost:${SERVER_PORT}`]
      })]
    }, PORT)

    const app = await createExpressServer(SERVER_PORT)
    const mockCallback = jest.fn()
    app.get('/index.js', mockCallback)

    const { data } = await axios({ baseURL: `http://localhost:${PORT}/index.js` })
    expect(data).toEqual('window.test = \'foo\'\n')

    // give it more time to make sure, the other servers would also been called
    await new Promise(resolve => setTimeout(resolve, 50))
    expect(mockCallback).not.toBeCalled()
  })

  it('caches the handler of a file and reuses it', async function () {
    await createViteServer({
      root: __dirname,
      logLevel: 'silent',
      plugins: [vitePluginProxy({
        frontendURIs: [`http://localhost:${SERVER_PORT}`, `http://localhost:${SERVER_PORT2}`]
      })]
    }, PORT)

    const app = await createExpressServer(SERVER_PORT)
    const mockCallback = jest.fn((req, res, next) => next())
    app.get('/other.js', mockCallback)

    const app2 = await createExpressServer(SERVER_PORT2)
    const mockCallback2 = jest.fn((req, res, next) => res.send('window.test = \'foo\'\n'))
    app2.get('/other.js', mockCallback2)

    const { data } = await axios({ baseURL: `http://localhost:${PORT}/other.js` })
    expect(data).toEqual('window.test = \'foo\'\n')
    expect(mockCallback).toBeCalledTimes(1)
    expect(mockCallback2).toBeCalledTimes(1)

    const { data: data2 } = await axios({ baseURL: `http://localhost:${PORT}/other.js` })
    expect(data2).toEqual('window.test = \'foo\'\n')
    expect(mockCallback).toBeCalledTimes(1)
    expect(mockCallback2).toBeCalledTimes(2)
  })

  describe('compression', function () {
    it('serves gzipped manifests', async function () {
      await createViteServer({
        root: __dirname,
        logLevel: 'silent',
        plugins: [vitePluginProxy({
          frontendURIs: [`http://localhost:${SERVER_PORT}`]
        }), vitePluginOxManifests({
          // no config
        })]
      }, PORT)

      const app = await createExpressServer(SERVER_PORT)
      app.use(compression({ threshold: 0 }))
      app.get('/manifests', (req, res) => {
        res.json([{ namespace: 'other' }])
      })

      const { data } = await axios({
        baseURL: `http://localhost:${PORT}/manifests`,
        headers: {
          'accept-encoding': 'gzip'
        }
      })

      expect(data).toEqual([{
        namespace: 'other'
      }, {
        namespace: 'index',
        path: 'index'
      }])
    })

    it('serves inflated manifests', async function () {
      await createViteServer({
        root: __dirname,
        logLevel: 'silent',
        plugins: [vitePluginProxy({
          frontendURIs: [`http://localhost:${SERVER_PORT}`]
        }), vitePluginOxManifests({
          // no config
        })]
      }, PORT)

      const app = await createExpressServer(SERVER_PORT)
      app.use(compression({ threshold: 0 }))
      app.get('/manifests', (req, res) => {
        res.json([{ namespace: 'other' }])
      })

      const { data } = await axios({
        baseURL: `http://localhost:${PORT}/manifests`,
        headers: {
          'accept-encoding': 'deflate'
        }
      })

      expect(data).toEqual([{
        namespace: 'other'
      }, {
        namespace: 'index',
        path: 'index'
      }])
    })
  })
})
