import { describe, it, expect, afterEach } from '@jest/globals'
import path from 'path'
import vitePluginProxy from '../../src/index'
import axios from 'axios'
import { closeAllServers, createExpressServer, createViteServer, getPort } from '../util'

const PORT = getPort()
const SERVER_PORT = getPort()
const __dirname = path.dirname(new URL(import.meta.url).pathname)

describe('simple proxy', function () {
  afterEach(async function () {
    await closeAllServers()
  })

  it('works', async function () {
    await createViteServer({
      root: __dirname,
      logLevel: 'silent',
      plugins: [vitePluginProxy({
        proxy: {
          '/api': {
            target: `http://localhost:${SERVER_PORT}`
          }
        }
      })]
    }, PORT)

    const app = await createExpressServer(SERVER_PORT)
    app.get('/api/test', (req, res) => {
      res.json({ hello: 'world' })
    })

    const { data } = await axios({ baseURL: `http://localhost:${PORT}/api/test` })
    expect(data).toEqual({ hello: 'world' })
  })

  it('regular expressions', async function () {
    await createViteServer({
      root: __dirname,
      logLevel: 'silent',
      plugins: [vitePluginProxy({
        proxy: {
          '^/(foo|bar)': {
            target: `http://localhost:${SERVER_PORT}`
          }
        }
      })]
    }, PORT)

    const app = await createExpressServer(SERVER_PORT)
    app.get('/foo/test', (req, res) => {
      res.json({ hello: 'foo' })
    })
    app.get('/bar/test', (req, res) => {
      res.json({ hello: 'bar' })
    })

    const { data } = await axios({ baseURL: `http://localhost:${PORT}/foo/test` })
    expect(data).toEqual({ hello: 'foo' })
    const { data: data2 } = await axios({ baseURL: `http://localhost:${PORT}/bar/test` })
    expect(data2).toEqual({ hello: 'bar' })
  })

  it('rewrite rule', async function () {
    await createViteServer({
      root: __dirname,
      logLevel: 'silent',
      plugins: [vitePluginProxy({
        proxy: {
          '/api': {
            target: `http://localhost:${SERVER_PORT}`,
            rewrite: () => '/appsuite/api/test'
          }
        }
      })]
    }, PORT)

    const app = await createExpressServer(SERVER_PORT)
    app.get('/appsuite/api/test', (req, res) => {
      res.json({ hello: 'world' })
    })

    const { data } = await axios({ baseURL: `http://localhost:${PORT}/api/test` })
    expect(data).toEqual({ hello: 'world' })
  })

  it('prepends a path', async function () {
    await createViteServer({
      root: __dirname,
      logLevel: 'silent',
      plugins: [vitePluginProxy({
        proxy: {
          '/api': {
            target: `http://localhost:${SERVER_PORT}`,
            prependPath: '/appsuite'
          }
        }
      })]
    }, PORT)

    const app = await createExpressServer(SERVER_PORT)
    app.get('/appsuite/api/test', (req, res) => {
      res.json({ hello: 'world' })
    })

    const { data } = await axios({ baseURL: `http://localhost:${PORT}/api/test` })
    expect(data).toEqual({ hello: 'world' })
  })
})
